import configparser
import os
import typing

from sitri.providers.base import ConfigProvider


class IniConfigProvider(ConfigProvider):
    """Config provider for Initialization file (Ini)."""

    provider_code = "ini"

    def __init__(
        self,
        ini_path: str = "./config.ini",
    ):
        """

        :param ini_path: path to ini file
        """
        self.configparser = configparser.ConfigParser()

        with open(os.path.abspath(ini_path)) as f:
            self.configparser.read_file(f)

        self._sections = None

    @property
    def sections(self):
        if not self._sections:
            self._sections = list(self.configparser.keys())

        return self._sections

    def get(self, key: str, section: str, **kwargs) -> typing.Optional[typing.Any]:
        """Get value from ini file.

        :param key: key or path for search
        :param section: section of ini file
        """
        if section not in self.sections:
            return None

        return self.configparser[section].get(key)

    def keys(self, section: str) -> typing.List[str]:
        """Get keys of section.

        :param section: section of ini file
        """
        if section not in self.sections:
            return []

        return list(self.configparser[section].keys())



from typing import Dict, Optional, Union

from hvac.exceptions import VaultError
from loguru import logger
from pydantic import Field
from pydantic.env_settings import SettingsError
from pydantic.main import BaseModel

from sitri.providers.contrib.json import JsonConfigProvider
from sitri.providers.contrib.vault import VaultKVConfigProvider
from sitri.settings.base import BaseLocalModeConfig, BaseLocalModeSettings


class VaultKVLocalProviderArgs(BaseModel):
    json_path: str = Field(...)
    default_path_mode_state: bool = Field(default=True)


class VaultKVSettings(BaseLocalModeSettings):
    @property
    def local_provider(self) -> JsonConfigProvider:
        if not self.__config__.local_provider:
            args = self.__config__.local_provider_args
            if args:
                if isinstance(args, dict):
                    args = VaultKVLocalProviderArgs(**args)

                self.__config__.local_provider = JsonConfigProvider(
                    json_path=args.json_path, default_path_mode_state=args.default_path_mode_state
                )
            else:
                raise ValueError("Local provider arguments not found for local mode")

        return self.__config__.local_provider

    def _build_local(self):
        d: Dict[str, Optional[str]] = {}

        provider = self.local_provider

        for field in self.__fields__.values():
            value: Optional[str] = None

            if self.__config__.local_mode_path_prefix:
                path = f"{self.__config__.local_mode_path_prefix}.{field.alias}"

            else:
                path = field.alias

            try:
                value = provider.get(key=path, separator=".")
            except VaultError:
                logger.opt(exception=True).warning(f"Could not get local variable {path}")

            if field.is_complex() and (
                isinstance(value, str) or isinstance(value, bytes) or isinstance(value, bytearray)
            ):
                try:
                    value = self.__config__.json_loads(value)  # type: ignore
                except ValueError as e:
                    raise SettingsError(f"Error parsing JSON for variable {path}") from e

            if value is None and field.default is not None:
                value = field.default

            d[field.alias] = value

        return d

    def _build_default(self):
        d: Dict[str, Optional[str]] = {}

        provider = self.__config__.provider

        for field in self.__fields__.values():
            vault_val: Optional[str] = None

            vault_secret_path = field.field_info.extra.get("vault_secret_path")
            vault_mount_point = field.field_info.extra.get("vault_mount_point")
            vault_secret_key = field.field_info.extra.get("vault_secret_key")

            if vault_secret_key is None:
                vault_secret_key = field.alias

            try:
                vault_val = provider.get(
                    key=vault_secret_key,
                    secret_path=vault_secret_path if vault_secret_path else self.__config__.default_secret_path,
                    mount_point=vault_mount_point if vault_mount_point else self.__config__.default_mount_point,
                )
            except VaultError:
                logger.opt(exception=True).warning(
                    f'Could not get secret "{vault_mount_point}/{vault_secret_path}:{vault_secret_key}"'
                )

            if field.is_complex() and (
                isinstance(vault_val, str) or isinstance(vault_val, bytes) or isinstance(vault_val, bytearray)
            ):
                try:
                    vault_val = self.__config__.json_loads(vault_val)  # type: ignore
                except ValueError as e:
                    raise SettingsError(
                        f'Error parsing JSON for "{vault_mount_point}/{vault_secret_path}:{vault_secret_key}"'
                    ) from e

            if vault_val is None and field.default is not None:
                vault_val = field.default

            d[field.alias] = vault_val

        return d

    class VaultKVSettingsConfig(BaseLocalModeConfig):
        provider: VaultKVConfigProvider
        default_secret_path: Optional[str] = None
        default_mount_point: Optional[str] = None

        local_mode: bool = False

        local_mode_path_prefix: Optional[str] = None
        local_provider_args: Optional[Union[VaultKVLocalProviderArgs, Dict]]

        local_provider: JsonConfigProvider = None

    __config__: VaultKVSettingsConfig



"""
.. module:: config.providers
   :synopsis: Config Base
.. moduleauthor:: Aleksander Lavrov <github.com/egnod>
"""


import typing
from abc import ABC, abstractmethod


class PathModeStateProvider(ABC):
    """Base class for config providers with path_mode discovery."""

    _default_path_mode_state: bool = False

    def _get_path_mode_state(self, path_mode_param: typing.Optional[bool]) -> bool:
        return path_mode_param if path_mode_param is not None else self._default_path_mode_state

    @abstractmethod
    def get(self, key: str, path_mode: typing.Optional[bool] = None, **kwargs) -> typing.Optional[typing.Any]:
        """Get value from storage.

        :param key: key for get value
        :param path_mode: find value by path with separated key
        :param kwargs: additional arguments for providers
        """

    @abstractmethod
    def _get_by_path(self, path: str, separator: str) -> typing.Any:
        """Retrieve value from a dictionary using a list of keys.

        :param path: string with separated keys
        """

    @abstractmethod
    def _get_by_key(self, key: str) -> typing.Any:
        """Retrieve value from a dictionary using a key.

        :param key: key from json
        """



from abc import ABC, abstractmethod
from pathlib import Path
from typing import Any, Dict, Optional, Type, Union

from pydantic import BaseConfig as PydanticBaseConfig
from pydantic import BaseSettings as PydanticBaseSettings
from pydantic.utils import deep_update

from sitri.providers.base import ConfigProvider


class BaseConfig(PydanticBaseConfig):
    provider: Type[ConfigProvider]


class BaseLocalModeConfig(BaseConfig):
    provider: Type[ConfigProvider]

    local_mode: Optional[bool]


class BaseSettings(ABC, PydanticBaseSettings):
    def _build_values(
        self,
        init_kwargs: Dict[str, Any],
        _env_file: Union[Path, str, None] = None,
        _env_file_encoding: Optional[str] = None,
        _secrets_dir: Union[Path, str, None] = None,
    ) -> Dict[str, Any]:
        return deep_update(
            deep_update(self._build_default()),
            init_kwargs,
        )

    @abstractmethod
    def _build_default(self, *args, **kwargs) -> Dict[str, Any]:
        pass

    __config__: Type[BaseConfig]


class BaseLocalModeSettings(BaseSettings, ABC):
    @property
    @abstractmethod
    def local_provider(self) -> Type[ConfigProvider]:
        pass

    @abstractmethod
    def _build_local(self, *args, **kwargs) -> Dict[str, Any]:
        pass

    def _build_values(
        self,
        init_kwargs: Dict[str, Any],
        _env_file: Union[Path, str, None] = None,
        _env_file_encoding: Optional[str] = None,
        _secrets_dir: Union[Path, str, None] = None,
    ) -> Dict[str, Any]:
        if not self.__config__.local_mode:
            return deep_update(
                deep_update(self._build_default()),
                init_kwargs,
            )
        else:
            return deep_update(deep_update(self._build_local()), init_kwargs)

    __config__: Type[BaseLocalModeConfig]



from typing import Dict, Optional

from pydantic.env_settings import SettingsError

from sitri.providers.contrib.yaml import YamlConfigProvider
from sitri.settings.base import BaseConfig, BaseSettings


class YamlSettings(BaseSettings):
    def _build_default(self):
        d: Dict[str, Optional[str]] = {}

        provider = self.__config__.provider

        for field in self.__fields__.values():
            key_name: Optional[str] = field.field_info.extra.get("yaml_key_name")
            path_prefix: Optional[str] = field.field_info.extra.get("yaml_path_prefix")

            if key_name is None:
                key_name = field.alias

            if path_prefix is not None:
                key_name = f"{path_prefix}{provider.separator}{key_name}"

            elif self.__config__.default_path_prefix is not None:
                key_name = f"{self.__config__.default_path_prefix}{provider.separator}{key_name}"

            value: Optional[str] = provider.get(key_name)

            if field.is_complex() and (
                isinstance(value, str) or isinstance(value, bytes) or isinstance(value, bytearray)
            ):
                try:
                    value = self.__config__.json_loads(value)  # type: ignore

                except ValueError as e:
                    raise SettingsError(f'Error parsing JSON for "{key_name}"') from e

            if value is None and field.default is not None:
                value = field.default

            d[field.alias] = value

        return d

    class YamlSettingsConfig(BaseConfig):
        provider: YamlConfigProvider

        default_path_prefix: Optional[str] = None

    __config__: YamlSettingsConfig



import os
import typing

from sitri.providers.base import ConfigProvider


class SystemConfigProvider(ConfigProvider):
    """Provider for get config from system environment."""

    provider_code = "system"
    _prefix = ""

    def __init__(self, prefix: typing.Optional[str] = None):
        """
        :param prefix: prefix for create "namespace" for project variables in environment
        """
        if prefix:
            self._prefix = prefix.upper()

    def prefixize(self, key: str) -> str:
        """Get key with prefix.

        :param key: varname without prefix
        """
        if self._prefix:
            return f"{self._prefix}_{key.upper()}"

        return key.upper()

    def unprefixize(self, var_name: str) -> str:
        """Remove prefix from variable name.

        :param var_name: variable name
        """
        if self._prefix:
            return var_name.replace(f"{self._prefix}_", "").lower()

        return var_name

    def get(self, key: str, **kwargs) -> typing.Union[str, None]:
        """Get value from system env.

        :param key: key without prefix from env
        """
        return os.getenv(self.prefixize(key), None)

    def keys(self) -> typing.List[str]:
        """Get keys list with prefix from system env."""
        var_list = []

        for var in os.environ:
            if var.upper().startswith(self._prefix):
                var_list.append(self.unprefixize(var))

        return var_list



import typing

import vedis
from loguru import logger

from sitri.providers.base import ConfigProvider


class VedisConfigProvider(ConfigProvider):
    """Config provider for vedis storage."""

    provider_code = "vedis"
    _hash_name = "sitri_config_hash"

    def __init__(self, vedis_connector: typing.Callable[[], vedis.Vedis], hash_name: str = "sitri_config_hash"):
        """

        :param vedis_connector: return connection to vedis
        :param hash_name: name for hash (key-value object) in vedis
        """
        self._vedis_get = vedis_connector
        self._hash_name = hash_name
        self._vedis_instance = None

    @property
    def _vedis(self) -> vedis.Vedis:
        if not self._vedis_instance:
            self._vedis_instance = self._vedis_get()

        return self._vedis_instance

    @property
    def _config_hash(self):
        return self._vedis.Hash(self._hash_name)

    @logger.catch(level="ERROR")
    def get(self, key: str, **kwargs) -> typing.Optional[str]:
        result = self._config_hash.get(key)

        if isinstance(result, bytes):
            return result.decode()

        return None

    @logger.catch(level="ERROR")
    def keys(self) -> typing.List[str]:
        var_list = []
        variables = self._config_hash.keys() if self._config_hash.keys() is not None else []

        for var in variables:
            var_list.append(var.decode())

        return var_list



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sitri',
 'sitri.providers',
 'sitri.providers.base',
 'sitri.providers.contrib',
 'sitri.providers.contrib.vault',
 'sitri.settings',
 'sitri.settings.contrib',
 'sitri.settings.contrib.vault',
 'sitri.strategy']

package_data = \
{'': ['*']}

install_requires = \
['loguru>=0.5.1,<0.6.0', 'pydantic>=1.7.3,<2.0.0']

extras_require = \
{'consul': ['python-consul>=1.1.0,<2.0.0'],
 'hvac': ['hvac>=0.10.5,<0.11.0'],
 'pyyaml': ['pyyaml>=5.3.1,<6.0.0'],
 'redis': ['redis>=3.5.3,<4.0.0'],
 'vedis': ['vedis>=0.7.1,<0.8.0']}

setup_kwargs = {
    'name': 'sitri',
    'version': '0.10.24',
    'description': 'Library for one endpoint config managment',
    'long_description': '\n<p align="center">\n  <a href="https://github.com/elastoo-team/sitri">\n    <img src="docs/_static/logo.gif">\n  </a>\n  <h1 align="center">\n    Sitri Configuration Library\n  </h1>\n</p>\n\n[![Build Status](https://img.shields.io/endpoint.svg?url=https%3A%2F%2Factions-badge.atrox.dev%2FElastoo-Team%2Fsitri%2Fbadge&style=popout)](https://actions-badge.atrox.dev/Elastoo-Team/sitri/goto)\n[![codecov](https://codecov.io/gh/Elastoo-Team/sitri/branch/master/graph/badge.svg)](https://codecov.io/gh/elastoo-team/sitri)\n![PyPI](https://img.shields.io/pypi/v/sitri)\n![Read the Docs](https://img.shields.io/readthedocs/sitri)\n[![CodeFactor](https://www.codefactor.io/repository/github/elastoo-team/sitri/badge)](https://www.codefactor.io/repository/github/elastoo-team/sitri)\n[![Maintainability](https://api.codeclimate.com/v1/badges/625f1d869adbf4128f75/maintainability)](https://codeclimate.com/github/Elastoo-Team/sitri/maintainability)\n\nSitri - library for managing authorization and configuration data from a single object with possibly different or identical providers\n\n#  Installation\n\n```bash\npoetry add sitri\n```\n\nor\n```bash\npip3 install sitri\n```\n\n# Basics with SystemProvider\n\n```python\nfrom sitri.providers.contrib import SystemConfigProvider\nfrom sitri import Sitri\n\nconf = Sitri(\n    config_provider=SystemConfigProvider(prefix="basics"),\n)\n```\nSystem provider use system environment for get config data. For unique - sitri lookup to "namespace" by prefix.\n\nExample:\n\n*In console:*\n```bash\nexport BASICS_NAME=Huey\n```\n\n*In code:*\n```python\nname = conf.get_config("name")\n\nprint(name)  # output: Huey\n```\n\n#  Docs\nRead base API references and other part documentation on https://sitri.readthedocs.io/\n',
    'author': 'Alexander Lavrov',
    'author_email': 'egnod@ya.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Egnod/sitri',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import typing

import hvac
from loguru import logger

from sitri.providers.base import ConfigProvider


class VaultKVConfigProvider(ConfigProvider):
    """Config provider for vault_kv storage.

    Only for kv first version at the moment.
    """

    provider_code = "vault_kv"

    def __init__(
        self,
        vault_connector: typing.Callable[[], hvac.Client],
        mount_point: typing.Optional[str] = None,
        secret_path: typing.Optional[str] = None,
    ):
        """
        :param vault_connector: function return connection to Vault
        :param mount_point: default vault_kv kv1 storage mount point
        :param secret_path: default path to the secret in mounted
        """

        self._vault_get = vault_connector
        self._vault_hvac_instance = None
        self._mount_point = mount_point
        self._secret_path = secret_path

    @property
    def _vault(self) -> hvac.Client:
        if not self._vault_hvac_instance:
            self._vault_hvac_instance = self._vault_get()

        return self._vault_hvac_instance

    @logger.catch(level="ERROR")
    def get(
        self, key: str, mount_point: typing.Optional[str] = None, secret_path: typing.Optional[str] = None, **kwargs
    ) -> typing.Optional[str]:
        request_params = {
            "path": secret_path if secret_path else self._secret_path,
            "mount_point": mount_point if mount_point else self._mount_point,
        }

        response = self._vault.secrets.kv.v1.read_secret(**request_params)

        return response["data"].get(key)

    @logger.catch(level="ERROR")
    def keys(
        self, mount_point: typing.Optional[str] = None, secret_path: typing.Optional[str] = None, **kwargs
    ) -> typing.List[str]:
        request_params = {
            "path": secret_path if secret_path else self._secret_path,
            "mount_point": mount_point if mount_point else self._mount_point,
        }

        response = self._vault.secrets.kv.v1.read_secret(**request_params)

        return list(response["data"].keys())
